import sys
import paho.mqtt.client as mqtt 
import time
import random
import string
import configparser
import db_util
import model
import message_util
from datetime import datetime


mqtt_server = "000.000.000.000"
mqtt_port = 1883
device_id = "Server_Background"

def on_connect(client, userdata, flags, rc):
    message_util.printLog(f"Connected with result code {rc}")
    #Actions sent by device ID
    client.subscribe("+/readtemp/#")
    client.subscribe("+/readdht22/#")
    client.subscribe("+/readinput/#")
    client.subscribe("+/readrpict3v1/#")
    client.subscribe("+/status/#")
    client.subscribe("+/statusin/#")


# the callback function, it will be triggered when receiving messages general
def on_message(client, userdata, msg):
    start_time = time.time()
    try:
        mensaje_string = msg.payload.decode("utf-8")
        separador = msg.topic.split("/")
        id_device = ""
        id_instalce=""
        action = ""
        app = ""
        gpio = ""
        #We divide the topic to obtain the data of the query and the team that performs it
        if(len(separador)>2):
            if("_" in str(separador[0])):
                id_device = separador[0].split("_")[0]                
                id_instalce= separador[0].split("_")[1]
            else:
                id_device = separador[0]
            action = separador[1]
            gpio = separador[2]
        else:
            return
        message=model.MessageData(msg,client)
        
        message_util.printLog(message.id_device+" "+ message.action)
        #We check if the team ID is authorized (it is in the BBDD)
        device_auth=db_util.autorize_device(message.id_device)
        if(not device_auth[0]):
            message_util.printLog("on_message | No Autorizado ID -->"+message.id_device)
            # Unauthorized equipment
            return
        # Authorized team
        device_registered = model.Device(message.id_device)
        # We look for the command to register the action
        for act in device_registered.device_actions:  
            if(act.pin_type=="readrpict3v1"):   
                print (mensaje_string)                      
            if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):# the action must be registered and the pin
                # the value is recorded
                last_value= db_util.get_aclion_last_value(device_registered.id,act.id, act.pin_number)
                # get the real value
                value = message_util.process_mensaje(message.action,message.mensaje_string, device_registered, act)
                if(value is not "ERROR" and value is not None):#comprobar el valor, si es error no se procesa
                    db_util.insert_action_value(device_registered.id,value,act.id,gpio)#lo insertamos en la base de datos
                    message_util.printLog("on_message | "+value+" ("+str(round((time.time() - start_time),4))+")")
    except Exception as ex:
        message_util.printLog("on_message | Exception SERVER ->"+ str(ex) )

# the callback function, it will be triggered when receiving messages
def on_message_read_rpict3v1(client, userdata, msg):
    start_time = time.time()
    try:
        message=model.MessageData(msg,client)
        device_auth=db_util.autorize_device(message.id_device)
        if(not device_auth[0]):
            message_util.printLog("read_rpict3v1 | No Autorizado ID -->"+message.id_device)
            return
        device_registered = model.Device(message.id_device)     
        for act in device_registered.device_actions:
            if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):#
                # the value is recorded
                last_value= db_util.get_aclion_last_value(device_registered.id,act.id, act.pin_number)
                #get the real value
                value = message_util.process_mensaje(message.action,message.mensaje_string, device_registered, act)                          
                if(value is not "ERROR" and value is not None):#comprobar el valor, si es error no se procesa
                    db_util.insert_action_value(device_registered.id,value,act.id,message.gpio)#lo insertamos en la base de datos
                    message_util.printLog("read_rpict3v1 | "+value+" ("+str(round((time.time() - start_time),4))+")")  
    except Exception as ex:
        message_util.printLog("read_rpict3v1 | Exception SERVER  -> "+ str(ex) )

# the callback function, it will be triggered when receiving messages
def on_message_status_device(client, userdata, msg):
    datetime_msg=str(datetime.now().strftime("%d/%m/%Y %H:%M:%S"))+"\t"
    try:
        message=model.MessageData(msg,client)
        #We check if the team ID is authorized (it is in the BBDD)
        device_auth=db_util.autorize_device(message.id_device)
        if(not device_auth[0]):
            message_util.printLog("status_device | No Autorizado ID -->"+message.id_device)
            #Unauthorized equipment
            return
        #Authorized equipment
        device_registered = model.Device(message.id_device)
        #we look for the command to register the action        
        for act in device_registered.device_actions:
            if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):#the action must be registered and the pin
                # the value is recorded
                last_value= db_util.get_aclion_last_value(device_registered.id,act.id, act.pin_number)
                #get the real value
                value = message_util.process_mensaje(message.action,message.mensaje_string, device_registered, act)   
                message_util.printLog("status_device | "+value)             
                if(value is not "ERROR" and value is not None):#comprobar el valor, si es error no se procesa
                    db_util.insert_action_value(device_registered.id,value,act.id,message.gpio)#lo insertamos en la base de datos
    except Exception as ex:
        message_util.printLog("status_device | Exception SERVER  -> "+ str(ex) )

# the callback function, it will be triggered when receiving messages
def on_message_read_input(client, userdata, msg):
    datetime_msg=str(datetime.now().strftime("%d/%m/%Y %H:%M:%S"))+"\t"    
    try:        
        message=model.MessageData(msg,client)
        #message_util.printLog(id_device+action)
        #We check if the team ID is authorized (it is in the BBDD)
        device_auth=db_util.autorize_device(message.id_device)
        if(not device_auth[0]):
            message_util.printLog("read_input | No Autorizado ID -->"+message.id_device)
            #Unauthorized equipment
            return
        #Equipo autorizado
        device_registered = model.Device(message.id_device)
        #we look for the command to register the action        
        for act in device_registered.device_actions:       
            if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):#the action must be registered and the pin
                
                # the value is recorded
                last_value= db_util.get_aclion_last_value(device_registered.id,act.id, act.pin_number)
                #get the real value
                value = message_util.process_mensaje(message.action,message.mensaje_string, device_registered, act)                
                if(value is not "ERROR" and value is not None):#comprobar el valor, si es error no se procesa
                    db_util.insert_action_value(device_registered.id,value,act.id,message.gpio)#lo insertamos en la base de datos
                    message_util.printLog("read_input | ACTION:"+message.action+" VALUE: "+str(value ))           
    except Exception as ex:
        message_util.printLog("read_input | Exception SERVER  -> "+ str(ex) )




#callback cuando recibimos mensaje de "ReadTemperature"
def on_message_read_temperature(client, userdata, msg):
    datetime_msg=str(datetime.now().strftime("%d/%m/%Y %H:%M:%S"))+"\t"    
    message=model.MessageData(msg,client)
    #correct data
    if(message.testCorrectData() is False):
        return    
    #autorizamos al dispositivo por BBDD
    device_auth=db_util.autorize_device(message.id_device)
    if(not device_auth[0]):
        message_util.printLog("read_temperature | No Autorizado ID -->"+message.id_device)
        #Unauthorized equipment
        return
    device_registered = model.Device(message.id_device)
    for act in device_registered.device_actions:       
        if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):#the action must be registered and the pin
        #valor de temperatura a tratar
            #get the real value
            value = message_util.process_mensaje(message.action,message.mensaje_string, device_registered, act)
            #obtenemos el valor de la media de la ultima hora natural de la base de datos
            value_avg=db_util.get_temperature_avg(device_registered.id, act.id, message.gpio)
            value_temperature_process=db_util.get_avg_float_values(value,value_avg)
            if(value_temperature_process is not "ERROR" and value_temperature_process is not None):#comprobar el valor, si es error no se procesa
                #We erase the values of that hour that we already have the average
                db_util.delete_hour_value(device_registered.id, act.id, message.gpio)
                #we insert the new value
                db_util.insert_action_value(device_registered.id,value_temperature_process,act.id,message.gpio)
                message_util.printLog("read_temperature | ACTION:"+message.action+" VALUE: "+str(value_temperature_process ))           


def on_message_read_dht22(client, userdata, msg):
    start_time = time.time()  
    message=model.MessageData(msg,client)
    #correct data
    if(message.testCorrectData() is False):
         return    
    #autorizamos al dispositivo por BBDD
    device_auth=db_util.autorize_device(message.id_device)
    if(not device_auth[0]):
        message_util.printLog("read_dht22 | No Autorizado ID -->"+message.id_device)
        #Unauthorized equipment
        return
    device_registered = model.Device(message.id_device)
    for act in device_registered.device_actions:    
        if(message.action==act.pin_type and int(message.gpio)==int(act.pin_number)):#the action must be registered and the pin
        #valor de temperatura a tratar
            #get the real value            
            value = message_util.process_dht22_sensor(message.action,message.mensaje_string, device_registered, act)            
            #we obtain the average value of the last calendar hour from the temperature and humidity database
            value_temp_avg=db_util.get_temperature_dht22_avg(device_registered.id, act.id, message.gpio)
            value_temperature_process=db_util.get_avg_float_values(value[0],value_temp_avg)

            value_hum_avg=db_util.get_humidity_dht22_avg(device_registered.id, act.id, message.gpio)
            value_hum_process=db_util.get_avg_float_values(value[1],value_temp_avg)

            #we treat temperature
            if(value_temperature_process is not "ERROR" and value_temperature_process is not None):#comprobar el valor, si es error no se procesa
                #We erase the values of that hour that we already have the average
                db_util.delete_hour_value(device_registered.id, act.id, message.gpio+"_temp")
                #we insert the new value
                db_util.insert_action_value(device_registered.id,value_temperature_process,act.id,message.gpio+"_temp")
                message_util.printLog("read_dht22 | temp "+str(value_temperature_process )+" ("+str(round((time.time() - start_time),4))+")")
            
            #we treat humidity
            if(value_hum_process is not "ERROR" and value_hum_process is not None):#comprobar el valor, si es error no se procesa
                #We erase the values of that hour that we already have the average
                db_util.delete_hour_value(device_registered.id, act.id, message.gpio+"_hum")
                #we insert the new value
                db_util.insert_action_value(device_registered.id,value_hum_process,act.id,message.gpio+"_hum")
                message_util.printLog("read_dht22 | hum "+str(value_hum_process )+" | ("+str(round((time.time() - start_time),4))+")")   
                #check CPDS alarms rules
                db_util.check_alarm_cpd(device_registered)         


client = mqtt.Client()
client.on_connect = on_connect
client.on_message = on_message

#We define the callback messages
client.message_callback_add("+/readtemp/#",on_message_read_temperature)
client.message_callback_add("+/readdht22/#",on_message_read_dht22)
client.message_callback_add("+/readinput/#",on_message_read_input)
client.message_callback_add("+/readrpict3v1/#",on_message_read_rpict3v1)
client.message_callback_add("+/status/#",on_message_status_device)

# create connection, the three parameters are broker address, broker port number, and keep-alive time respectively
client.connect(mqtt_server, mqtt_port, 3600)

client.loop_forever()