import db_util
import json
from datetime import datetime


"""
Process message. This method redirects the messages to their 
respective processing method, if something is not correct it 
returns ERROR and the message is not processed
"""
def process_mensaje(action , mensaje , device, action_obj):
    if(action=="readtemp"):
        return process_temperature(mensaje)
    if(action=="readinput"):
        #We process the value to know if it is a float, if it is not, it is not recorded
        return precess_input(action ,mensaje, device, action_obj)
    if(action=="readrpict3v1"):
        return process_readrpict3v1(action ,mensaje, device, action_obj)

    if(action=="action"):
        try:
            if(str(mensaje) == "ON" ):                
                return "ON"
            elif (str(mensaje) == "OFF"):
                return "OFF"
            else:
                return "ERROR"
        
        except Exception as ex:
            return "ERROR" #This value is not processed


"""
Process RPICT3v1 board message
"""
def process_readrpict3v1(action , mensaje , device, action_obj):
    try:
        json_string = json.dumps(eval(mensaje))
        obj = json.loads(json_string)        
        RealPower1=obj["realpower"][0]
        RealPower2=obj["realpower"][1]
        RealPower3=obj["realpower"][2]
        
        AppaPower1=obj["AppaPower"][0]
        AppaPower2=obj["AppaPower"][1]
        AppaPower3=obj["AppaPower"][2]
        
        irms1=obj["irms"][0]
        irms2=obj["irms"][1]
        irms3=obj["irms"][2]

        vrms1=obj["vrms"][0]
        vrms2=obj["vrms"][1]
        vrms3=obj["vrms"][2]

        vrms1=obj["powerfact"][0]
        vrms2=obj["powerfact"][1]
        vrms3=obj["powerfact"][2]
        return(irms1+","+irms2+","+irms3)
    except Exception as ex:
        return "ERROR" #This value is not processed


"""
Process input message
"""
def precess_input(action , mensaje , device, action_obj):
    try:
        last_value= db_util.get_aclion_last_value(device.id,action_obj.id, action_obj.pin_number)
        if(len(last_value)==0):#if the value is new
            if(str(mensaje) == "ON" ):                
                return "ON"
            elif (str(mensaje) == "OFF"):
                return "OFF"
            else:
                return "ERROR"
        else:#if the value exists, it is only logged if it has changed
            if(str(mensaje) == "ON" and str(last_value) == "OFF"):            
                return "ON"
            elif (str(mensaje) == "OFF" and str(last_value) == "ON"):
                return "OFF"
            else:
                return "ERROR"
    except Exception as ex:
            return "ERROR" #This value is not processed

"""
Process DHT22 sensor message
"""
def process_dht22_sensor(action , mensaje , device, action_obj):
    try:
        if(action=="readdht22"):
            return process_dht22(mensaje)
    except Exception as ex:
        return "ERROR" #This value is not processed


def process_temperature(mensaje):    
    #We process the value to know if it is a float, if it is not, it is not recorded        
    try:
        tempValue = 0.0 
                  
        tempValue=round(float(mensaje),2)
        
        if(tempValue>-50.0 and tempValue<100.0):
            #is valid
            return tempValue
        else:
            return "ERROR" #This value is not processed
    except:
        return "ERROR" #This value is not processed

"""
Process DHT22 sensor message
"""
def process_dht22(mensaje):    
    #We process the value to know if it is a float, if it is not, it is not recorded        
    try:
        obj = json.loads(mensaje)
        tempValue=round(float(obj["temp"]),2)
        humValue=round(float(obj["hum"]),2)
        if(tempValue>-50.0 and tempValue<100.0):
            pass
        else:
            tempValue="ERROR"
        if(humValue>-10.0 and humValue<120.0):
            pass
        else:
            humValue="ERROR"
        return [tempValue, humValue]
    except Exception as ex:
        return "ERROR" #This value is not processed



def printLog(mensaje):
    datetime_msg=str(datetime.now().strftime("%d/%m %H:%M:%S"))+" | "
    print(datetime_msg+str(mensaje))