import sys
import time
import RPi.GPIO as GPIO
import glob
import adafruit_dht 
import json 
import serial
from datetime import datetime


#define a GPIO as output
def gpio_Salida(numero):
    GPIO.setmode(GPIO.BCM)	
    GPIO.setwarnings(False)
    GPIO.setup(numero,GPIO.OUT)

def gpio_Entrada(numero):
    GPIO.setmode(GPIO.BCM)	
    GPIO.setwarnings(False)
    GPIO.setup(numero,GPIO.IN)

#turn on a GPIO
def encenderGPIO(numero):
	gpio_Salida(numero)
	GPIO.output(numero,0)

#turn off a GPIO
def apagarGPIO(numero):
	gpio_Salida(numero)
	GPIO.output(numero,1)


#read Input configured as output
def leerEntradaGPIO(numero):
	estado = GPIO.input(numero)
	if (estado == 0):
		return "ON"
	elif (estado == 1):
		return "OFF"
	else:
		return "ERROR"


#read Entry
def leerGPIO(numero):
	gpio_Entrada(numero)
	estado = GPIO.input(numero)
	if (estado == 0):
		return "ON"
	elif (estado == 1):
		return "OFF"
	else:
		return "ERROR"

def leerSensor(direccion):
	tfile=open(direccion)
	text = tfile.read()
	tfile.close()
	secondline = text.split("\n")[1]
	temperaturedata = secondline.split(" ")[9]
	temperature = float(temperaturedata[2:])
	temperature = temperature / 1000
	return float(temperature)	

def getIdSensors():
	sensorIds ={}
	for sensor in glob.glob("/sys/bus/w1/devices/28-*/w1_slave"):
		try:
			temp = leerSensor(sensor)
			sensorIds[sensor.split("/")[5]]=temp
		except Exception as ex:
			print(ex)
	return sensorIds

def getDHT22(sensorPin):
	sensor = adafruit_dht.DHT22(sensorPin) #Change for DHT22 and if you use that sensor
	try:		
		temperatura = sensor.temperature
		humedad = sensor.humidity
		print ("Humedad: " , humedad)
		print ("Temperatura: " , temperatura)
		dictionary='{"temp":"'+str(round(temperatura,2))+'","hum":"'+str(round(humedad,2))+'"}'
		sensor.exit()
		return (dictionary)
	except:
		sensor.exit()
		return ("ERROR")
	sensor.exit()

def get_temperature_media():
    temps= getIdSensors()
    return get_media(temps)

def get_media(temps):
    # arithmetic average
    values = []
    if(type(temps) is dict):# various sensors
        for key,value in temps.items():
            values.append(value)
    suma = 0
    for value in values:
        suma += value
    logitude = len(values)
    return suma / logitude


def getRpict3v1():
	ser = serial.Serial('/dev/ttyAMA0', 38400)
	try:
		line = ser.readline()
		line = line[:-2]
		Z = line.decode("utf-8").split(' ')
		"""ID Z[0]
		RealPower:Z[1], Z[6], Z[11]
		AppaPower:(Z[2], Z[7], Z[12]
		Irms     :(Z[3], Z[8], Z[13]
		Vrms     :(Z[4], Z[9], Z[14]
		PowerFact:(Z[5], Z[10], Z[15]
		"""
		values={
			"realpower":[Z[1], Z[6], Z[11]],
			"AppaPower":[Z[2], Z[7], Z[12]],
			"irms"     :[Z[3], Z[8], Z[13]],
			"vrms"     :[Z[4], Z[9], Z[14]],
			"powerfact":[Z[5], Z[10], Z[15]],
			"result":"OK"
		}
		return values
	except Exception as ex:
		return {"result":"ERROR"}
	finally:
		ser.close()

def printLog(mensaje):
	datetime_msg=str(datetime.now().strftime("%d/%m %H:%M:%S"))+" | "
	print(datetime_msg+str(mensaje))