
import sys
import paho.mqtt.client as mqtt 
import time , threading
import gpioUtil as util
import random
import string
import configparser
import json
from datetime import datetime
import system_read_config as configuration




#DATA
DEFAULT_CONFIR_ROUTE="/root/IOT_SYS/"
DEFAULT_CONFIR_FILE="config_iot_sys.ini"

#CONFIG SYSTEM
config = configparser.ConfigParser()
config.read(DEFAULT_CONFIR_ROUTE+DEFAULT_CONFIR_FILE)

#MQTT CONFIG
mqtt_server =configuration.mqtt_server
mqtt_port = configuration.mqtt_port
device_id = configuration.device_id
register_time_publish = configuration.register_time_publish
topics_suscribe = configuration.topics_suscribe


#Callback when connecting the client to the broker
def on_connect(client, userdata, flags, rc):
    print(f"Connected with result code {rc}")
    client.publish(device_id+"/status", payload=b'{"status": "on"}', qos=1, retain=True)
    for topic in topics_suscribe:
        client.subscribe(device_id+"/"+topic+"/#")
        print (device_id+"/"+topic+"/#")

#Callback when disconnecting the client to the broker
def on_disconnect(client, userdata, rc):
    tryToConnect=True
    try:
        while tryToConnect:
            time.sleep(10)
            print("disconected, try to connect reason-->"+str(rc))
            client.connect(mqtt_server, mqtt_port, keepalive=30)
            tryToConnect=False            
    except:
        print("connection failed")
    

# the callback function, it will be triggered when receiving messages
def on_message(client, userdata, msg):
    try:
        # Get the message and spread it
        mensaje_string = msg.payload.decode("utf-8")
        separador = msg.topic.split("/")
        if(len(separador)>1):
            if("_" in str(separador[0])):
                id_device = separador[0].split("_")[0]            
                id_instalce= separador[0].split("_")[1]
            else:
                id_device = separador[0]
            action = separador[1]
            gpio = separador[2]  
        
        print(str(separador)+" "+str(userdata)+" "+str(client._client_id))
        
        #We see if the team is the one to which the message is sent
        if(device_id==id_device):
            #Let's see if it's in command which means it's something to turn on or off
            if(action=="action"):
                #We use the message to turn equipment on or off
                if(mensaje_string=="ON"):
                    util.encenderGPIO(int(gpio))
                    client.publish(id_device+"/statusin/"+gpio, payload=util.leerEntradaGPIO(int(gpio)), qos=1, retain=True)
                    print ("ENCENDIDO  -->"+gpio)
                    #record action in database
                elif (mensaje_string=="OFF"):
                    util.apagarGPIO(int(gpio))
                    client.publish(id_device+"/statusin/"+gpio, payload=util.leerEntradaGPIO(int(gpio)), qos=1, retain=True)       
                    print ("APAGADO  -->"+gpio)         
                    #record action in database
                elif (mensaje_string=="READOUTPUT"):
                    print(util.leerEntradaGPIO(int(gpio)))
                    client.publish(id_device+"/statusin/"+gpio, payload=util.leerEntradaGPIO(int(gpio)), qos=0, retain=False)
                    print ("LECTURA SALIDA  -->"+gpio +" VALOR --> "+util.leerEntradaGPIO(int(gpio))) 
                    #record action in database
                elif (mensaje_string=="READTEMP"):
                    temps=util.getIdSensors()
                    temps=util.get_temperature_media()
                    client.publish(id_device+"/readtemp/"+gpio, payload=str(temps), qos=0, retain=False)
                elif (mensaje_string=="READINPUT"):
                    print(util.leerGPIO(int(gpio)))
                    client.publish(id_device+"/statusin/"+gpio, payload=util.leerGPIO(int(gpio)), qos=0, retain=False)
                elif (mensaje_string=="READDHT22"):
                    valuesDHT22=util.getDHT22(int(gpio))
                    print(str(valuesDHT22))
                    client.publish(id_device+"/readdht22/"+gpio, payload=str(valuesDHT22), qos=0, retain=False)
    except Exception as ex:
        print(str(ex))

client = mqtt.Client(client_id=(device_id+"_"), clean_session=True, userdata=None, transport="tcp")
client.on_connect = on_connect
client.on_disconnect = on_disconnect


client.on_message = on_message

# Set the will message, when the computer is off or the network is abnormally 
# interrupted, it will send the will message to other clients to ensure the connection status
client.will_set(device_id+'/status', b'{"status": "Off"}', qos=1, retain=True)


tryToConnect=True

# create connection, the three parameters are broker address, port number, and hold time
while tryToConnect:
    try:
        print("Try to connect")
        time.sleep(10)
        client.connect(mqtt_server, mqtt_port, keepalive=30)
        tryToConnect=False        
    except:
        print("connection failed")

# set the network loop lock, will not actively end the program before calling disconnect () or the program hangs, does not continue code execution
client.loop_forever()




