from django.db import models
from django.contrib.auth.models import User
from django.utils.text import slugify 
import paho.mqtt.publish as publish
from django.dispatch import receiver
from django.db.models.signals import post_save
from iot_django.models.aplication import *




 
#borrar imagenes
def custom_upload_to(instance, filename):
    old_instance = Device.objects.get(pk=instance.pk)
    old_instance.device_image.delete()
    return 'devices/' + filename


class Device(models.Model):
    device_name = models.CharField(max_length=100,verbose_name="Nombre del dispositivo")
    device_alias = models.CharField(max_length=200,verbose_name="Alias del dispositivo")
    device_location_latitude = models.CharField(max_length=70, default=None, blank=True, null=True,verbose_name="Latitud")
    device_location_longitude = models.CharField(max_length=70, default=None, blank=True, null=True,verbose_name="Longitud")
    device_MQTT_ID = models.CharField(max_length=100,verbose_name="Id MQTT",unique=True)
    device_user = models.ForeignKey(User, null=True,blank=True, on_delete=models.SET_NULL,verbose_name="Usuario")
    device_description = models.CharField(max_length=500,verbose_name="Descripcion")
    device_is_active = models.IntegerField(default=0,verbose_name="Dispositivo activo") #0-->Not active, 1 --> active    
    device_created_at =models.DateTimeField(auto_now_add=True,verbose_name="Creación")
    device_updated_at =models.DateTimeField(auto_now=True,verbose_name="Actualización")  
    slug = models.SlugField(max_length=40,unique=True, default="a", editable=False)
    device_image = models.ImageField(verbose_name="Imagen dispositivo", upload_to=custom_upload_to, default="devices/ESP8266_NodeMCU.png")
    last_ip_connection = models.CharField(max_length=200,null=True,blank=True,verbose_name="Ultima IP conocida")
    last_known_status = models.CharField(max_length=10,default="unknown", verbose_name="Estado")
    
    def save(self, *args, **kwargs):
        self.slug = slugify(self.device_MQTT_ID)
        super(Device, self).save(*args, **kwargs)
    class Meta:
        verbose_name="dispositivo"
        verbose_name_plural="dispositivos"
        ordering= ["-device_updated_at"]
    def __str__(self):
        return self.device_alias


class Action_type(models.Model):
    action_type_topic=models.CharField(primary_key=True,null=False,max_length=50,verbose_name="Tipo de acción")
    action_type_description=models.CharField(max_length=200,default="",verbose_name="Descripción")
    def __str__(self):
        return self.action_type_description
    class Meta:
        verbose_name="Acciones"
        verbose_name_plural="Tipos de acciones"

class Action(models.Model):
    PINS_GPIO_RASPBERRY = [#action/+
        (17, "GPIO17"),
        (27, "GPIO27"),
        (22,"GPIO22"),
        (5,"GPIO5"),
        (6,"GPIO6"),
        (13,"GPIO13"),
        (19,"GPIO19"),
        (23,"GPIO23"),
        (24,"GPIO24"),
        (25,"GPIO25"),
        (12,"GPIO12"),
        (16,"GPIO16"),
        (20,"GPIO20"),
        (21,"GPIO21"),
        (4,"GPIO4-WIRE"),
        (100,"serialRpic")
    ]
    pin_name=models.CharField(max_length=200,verbose_name="Nombre de la acción")
    pin_description = models.CharField(max_length=500,verbose_name="Descripción")
    pin_created_at = models.DateTimeField(auto_now_add=True)
    pin_updated_at = models.DateTimeField(auto_now=True)
    pin_number= models.IntegerField(default=17,choices=PINS_GPIO_RASPBERRY,verbose_name="Nº Pin")
    pin_type = models.ForeignKey(Action_type, null=True, on_delete=models.CASCADE,verbose_name="Tipo accion")
    device_device = models.ForeignKey( Device, null=True, on_delete=models.CASCADE,verbose_name="Dispositivo asociado")
    aplication = models.ForeignKey(Application,null=True, blank=True,on_delete=models.SET_NULL,verbose_name="Aplicacion Asociada" )
    def __str__(self):
        return self.pin_name
    class Meta:
        verbose_name="action"
        verbose_name_plural="actions"
    # def publishON(self):
    #     publish.single("123456789a/action/16", "ON", hostname="192.168.1.21")
    # def publishOFF(self):
    #     publish.single("123456789a/action/16", "OFF", hostname="192.168.1.21")
    

class Action_values(models.Model):
    action_values_device_id = models.CharField(max_length=250)
    action_values_discriminate = models.CharField(max_length=10)
    action_values_values = models.CharField(max_length=250)
    action_values_created_at = models.DateTimeField(auto_now=True)
    action_values_action= models.ForeignKey(Action, null=True, on_delete=models.CASCADE)
    class Meta:
        ordering = ['-action_values_created_at']





class Status_values(models.Model):
    status_value_device_id = models.CharField(max_length=250)
    status_value_discriminate = models.CharField(max_length=250)
    status_value_value = models.CharField(max_length=250)
    status_value_updated_at = models.DateTimeField(auto_now=True)
    status_device = models.ForeignKey( Device, null=True, on_delete=models.CASCADE)

