from django.shortcuts import render , redirect
from django.contrib.auth import authenticate, login
from django.views.generic import ListView, DetailView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView , UpdateView, DeleteView
from django.views.generic.list import ListView 
from iot_django.models.device import *
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic.edit import FormMixin
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import permission_required,login_required
from iot_django.forms import *
from django.http import JsonResponse
import logging

from datetime import datetime, timedelta




@login_required
def getDataElectricity(request):
    days = int(request.GET.get('days', None))
    action_id = request.GET.get('action_id', None)
    action= Action.objects.filter(id=action_id).first()
    data = {}
    values1=[]
    values2=[]
    values3=[]
    for value in action.action_values_set.all().filter(action_values_created_at__gte=datetime.now() - timedelta(days=(days))):
        timestamp=value.action_values_created_at.timestamp() * 1000
        values1.append([timestamp, float(value.action_values_values.split(',')[0])])
        values2.append([timestamp, float(value.action_values_values.split(',')[1])])
        values3.append([timestamp, float(value.action_values_values.split(',')[2])])
    data["data_type"]="electricity"
    data["sensor1"]=values1
    data["sensor2"]=values2
    data["sensor3"]=values3
    return JsonResponse(data)

@login_required
def getDataTemHum(request):
    days = int(request.GET.get('days', None))
    action_id = request.GET.get('action_id', None)
    action= Action.objects.filter(id=action_id).first()
    data = {}
    valuesTemp=[]
    valuesHum=[]
    for value in action.action_values_set.all().filter(action_values_created_at__gte=datetime.now() - timedelta(days=(days))):
        timestamp=value.action_values_created_at.timestamp() * 1000
        if("_temp" in value.action_values_discriminate):
            valuesTemp.append([timestamp, float(value.action_values_values)])
        if("_hum" in value.action_values_discriminate):
            valuesHum.append([timestamp, float(value.action_values_values)])
    data["data_type"]="TempHum"
    data["Temp"]=valuesTemp
    data["Hum"]=valuesHum
    return JsonResponse(data)




class ActionDetailView (DetailView):
    model = Action 
    template_name = 'action/action.html'
    success_url = reverse_lazy('action')

    
    



@method_decorator(staff_member_required,'dispatch')
class ActionCreate(CreateView):
    model = Action
    form_class= ActionCreateForm
    template_name = 'action/action_create.html'
    #fields = '__all__'#['device_name', 'device_alias', 'device_MQTT_ID', 'device_description']
    success_url = reverse_lazy('devices')

    def get_success_url(self):
        id_dev = self.kwargs['device_id']
        return reverse_lazy('device', args=[id_dev]) + '?ok'

    def form_valid(self, form ):
        self.object = form.save(commit=False)
        id_dev = self.kwargs['device_id']#int(self.request.build_absolute_uri('/').strip("/")[2])
        self.object.device_device = Device.objects.get(id=id_dev)
        self.object.save()
        return super(ActionCreate, self).form_valid(form)



@method_decorator(staff_member_required,'dispatch')
class ActionDelete(DeleteView):
    model = Action
    template_name = 'action/action_confirm_delete.html'
    success_url = reverse_lazy('devices')


@method_decorator(staff_member_required,'dispatch')
class ActionUpdate(UpdateView):
    model = Action
    template_name = 'action/action_update.html'
    fields = '__all__'#['name']    
    def get_success_url(self):
        return reverse_lazy('actionUpdate', args=[self.object.id]) + '?ok'