from django.shortcuts import render , redirect
from django.contrib.auth import authenticate, login
from django.views.generic import ListView, DetailView
from iot_django.models.device import *
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import permission_required,login_required
from django.views.generic.list import ListView 
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView , UpdateView, DeleteView
from iot_django.forms import DeviceForm, DeviceCreateForm


@login_required
def allDevices(request):
    devices = Device.objects.all()
    context={
        'devices':devices
    }    
    return render(request, "device/devices.html",context)

@login_required
def Device_detail(request, id):
    device = Device.objects.get(id=id)
    context={
        'device':device
    } 
    return render(request, "device/device.html",context)


class DeviceListView (ListView):
    model = Device 
    template_name = 'device/devices.html'
    paginate_by = 10

    def get_queryset(self):
        user=self.request.user
        user_device = Device.objects.filter(device_user_id__groups__in=user.groups.all()).distinct()
        return user_device
     
class DeviceDetailView (DetailView):
    model = Device 
    template_name = 'device/device.html'

@method_decorator(staff_member_required,'dispatch')
class DeviceCreate(CreateView):
    model = Device    
    form_class= DeviceCreateForm
    template_name = 'device/device_create.html'
    #fields = '__all__'#['device_name', 'device_alias', 'device_MQTT_ID', 'device_description']
    success_url = reverse_lazy('devices')

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.device_user = self.request.user
        self.object.save()
        return super(DeviceCreate, self).form_valid(form)
        
@method_decorator(staff_member_required,'dispatch')
class DeviceUpdate(UpdateView):
    model = Device
    form_class= DeviceForm
    template_name = 'device/device_update.html'
    #fields = '__all__'#['name']
    def get_success_url(self):
        return reverse_lazy('deviceUpdate', args=[self.object.id]) + '?ok'

@method_decorator(staff_member_required,'dispatch')
class DeviceDelete(DeleteView):
    model = Device
    template_name = 'device/device_confirm_delete.html'
    success_url = reverse_lazy('devices')